﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Microsoft.Research.Kinect.Nui;

namespace KinectWeatherMap
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        ImageFrame lastDepthFrame;
        int[] colorOffsetMap;

        public MainWindow()
        {
            InitializeComponent();

            this.Loaded += (s, e) =>
            {
                InitKinect(Runtime.Kinects.FirstOrDefault());
            };
        }

        void InitKinect(Runtime runtime)
        {
            if (runtime == null)
                return;

            runtime.Initialize(RuntimeOptions.UseColor |
                               RuntimeOptions.UseDepthAndPlayerIndex |
                               RuntimeOptions.UseSkeletalTracking);

            runtime.DepthFrameReady += runtime_DepthFrameReady;
            runtime.VideoFrameReady += runtime_VideoFrameReady;
            runtime.SkeletonFrameReady += runtime_SkeletonFrameReady;

            runtime.VideoStream.Open(ImageStreamType.Video,
                                     2,
                                     ImageResolution.Resolution640x480,
                                     ImageType.Color);
            runtime.DepthStream.Open(ImageStreamType.Depth,
                                     2,
                                     ImageResolution.Resolution320x240,
                                     ImageType.DepthAndPlayerIndex);

            colorOffsetMap = new int[320 * 240 * 2];
            var view = new ImageViewArea();
            var cam = runtime.NuiCamera;
            for (int y = 0; y < 240; y++)
            {
                for (int x = 0; x < 320; x++)
                {
                    int colorX;
                    int colorY;
                    cam.GetColorPixelCoordinatesFromDepthPixel(ImageResolution.Resolution640x480,
                                                               view, x, y, 0,
                                                               out colorX, out colorY);
                    colorOffsetMap[2 * (x + y * 320)] = colorX;
                    colorOffsetMap[2 * (x + y * 320) + 1] = colorY;
                }
            }

        }

        void runtime_DepthFrameReady(object sender, ImageFrameReadyEventArgs e)
        {
            lastDepthFrame = e.ImageFrame;
        }

        void runtime_VideoFrameReady(object sender, ImageFrameReadyEventArgs e)
        {
            if (lastDepthFrame == null)
                return;
            var depthImage = lastDepthFrame.Image;

            var rgbImage = e.ImageFrame.Image;

            byte[] foregroundImage = new byte[depthImage.Width * depthImage.Height * 4];

            var depthBytes = depthImage.Bits;
            var colorBytes = rgbImage.Bits;

            int depthWidth = depthImage.Width;
            int depthHeight = depthImage.Height;
            int rgbWidth = rgbImage.Width;
            int rgbHeight = rgbImage.Height;

            for (int y = 0; y < depthHeight; y++)
            {
                for (int x = 0; x < depthWidth; x++)
                {
                    int depthIndex = 2 * (x + y * depthWidth);

                    //Not needed for this project, but you might want to use depth as well:
                    //short depth = (short)((depthBytes[depthIndex] >> 3) |
                    //                      (depthBytes[depthIndex + 1] << 5));


                    int playerIndex = depthBytes[depthIndex] & 7;
                    if (playerIndex == 0)
                    {
                        continue;
                    }

                    int colorX = colorOffsetMap[depthIndex];
                    int colorY = colorOffsetMap[depthIndex + 1];

                    if (colorX < 0 || colorX >= rgbWidth ||
                        colorY < 0 || colorY >= rgbHeight)
                        continue;

                    int colorIndex = 4 * (colorX + colorY * rgbWidth);
                    int foregroundIndex = 2 * depthIndex;

                    foregroundImage[foregroundIndex + 0] = colorBytes[colorIndex + 0];
                    foregroundImage[foregroundIndex + 1] = colorBytes[colorIndex + 1];
                    foregroundImage[foregroundIndex + 2] = colorBytes[colorIndex + 2];
                    foregroundImage[foregroundIndex + 3] = 255;

                }
            }

            var source = BitmapSource.Create(depthImage.Width, depthImage.Height,
                                                96, 96,
                                                PixelFormats.Bgra32, null,
                                                foregroundImage, depthImage.Width * 4);
            kinectImage.Source = source;
        }

        void runtime_SkeletonFrameReady(object sender, SkeletonFrameReadyEventArgs e)
        {

        }

    }
}
